from __future__ import annotations

import argparse
import sys
from importlib.metadata import PackageNotFoundError, version
from pathlib import Path

from pydantic import BaseModel

from .anonymize import anonymize_package
from .get_packages import PyprojectTomlNotFoundError, get_packages
from .logger import logger
from .prepare_outdir import prepare_outdir


def get_version(name: str) -> str:
    try:
        return version(name)
    except PackageNotFoundError:
        return 'unknown'


class Options(BaseModel):
    output: Path
    exclude: list[str]

    @staticmethod
    def from_commandline() -> Options:
        parser = argparse.ArgumentParser(
            prog='unname',
            description='Anonymize Python codebases for blind review.',
        )
        parser.add_argument(
            '-o', '--output', type=Path, required=True, help='output directory'
        )
        parser.add_argument(
            '--exclude',
            type=str,
            nargs='*',
            default=['**/LICEN[CS]E', '**/LICEN[CS]E.*'],
            help='glob patterns to exclude from output (default: %(default)s)',
        )
        parser.add_argument(
            '-v',
            '--version',
            action='version',
            version=f'{parser.prog} {get_version(parser.prog)}',
        )
        args = parser.parse_args()

        options = Options.model_validate(args, from_attributes=True)
        return options


def main():
    options = Options.from_commandline()
    prepare_outdir(options.output, options.exclude)

    try:
        packages = get_packages(options.output)
    except PyprojectTomlNotFoundError as e:
        logger.error(e)
        return sys.exit(1)

    for package in packages:
        anonymize_package(package)


if __name__ == '__main__':
    main()



import logging

# Source - https://stackoverflow.com/a/56944256
# Posted by Sergey Pleshakov, modified by community. See post 'Timeline' for change history
# Retrieved 2025-11-21, License - CC BY-SA 4.0
# Further modified by Ryota Ushio to fix type errors


class CustomFormatter(logging.Formatter):
    grey = '\x1b[38;20m'
    yellow = '\x1b[33;20m'
    red = '\x1b[31;20m'
    bold_red = '\x1b[31;1m'
    reset = '\x1b[0m'
    template = '%(levelname)s: %(message)s'

    FORMATS = {
        logging.DEBUG: grey + template + reset,
        logging.INFO: grey + template + reset,
        logging.WARNING: yellow + template + reset,
        logging.ERROR: red + template + reset,
        logging.CRITICAL: bold_red + template + reset,
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


logger = logging.getLogger('unname')
logger.setLevel(logging.DEBUG)

# create console handler with a higher log level
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)

ch.setFormatter(CustomFormatter())

logger.addHandler(ch)



from .anonymize import (
    anonymize_package as anonymize_package,
    anonymize_pyproject_toml as anonymize_pyproject_toml,
    anonymize_readme_md as anonymize_readme_md,
)
from .get_packages import get_packages as get_packages



import shutil
import subprocess
from pathlib import Path


def prepare_outdir(outdir: Path, exclude_globs: list[str]):
    cleanup_outdir(outdir)
    copy_tracked_files(outdir, exclude_globs)


def cleanup_outdir(outdir: Path):
    if outdir.exists():
        shutil.rmtree(outdir)


def get_tracked_files() -> list[str]:
    tracked_files = subprocess.run(
        ['git', 'ls-files'],
        check=True,
        capture_output=True,
        text=True,
    ).stdout.splitlines()
    return tracked_files


def get_excluded_paths(exclude_globs: list[str]) -> set[str]:
    excluded_paths = set(
        str(path)
        for exclude_glob in exclude_globs
        for path in Path('.').glob(exclude_glob)
    )
    return excluded_paths


def copy_tracked_files(outdir: Path, exclude_globs: list[str]):
    tracked_files = get_tracked_files()
    excluded_paths = get_excluded_paths(exclude_globs)

    for file in tracked_files:
        if file in excluded_paths:
            continue
        outfile = outdir / file
        outfile.parent.mkdir(parents=True, exist_ok=True)
        shutil.copy(file, outfile)
